<?php
/*
 * Import files from one course to another
 */

require_once '../../config.php';
require $CFG->libdir.'/filelib.php';
require_once 'access_control.php';


////// FILE FUNCTIONS //////

/**
 * Set the list of files to be displayed
 * @global object $USER
 * @param array $VARS
 * @return int The number of files/folders
 */
function setfilelist($VARS) {
    global $USER;

    $USER->filelist = array ();
    $USER->fileop = "";

    $count = 0;
    foreach ($VARS as $key => $val) {
        if (substr($key,0,4) == "file") {
            $count++;
            $val = rawurldecode($val);
            $USER->filelist[] = clean_param($val, PARAM_PATH);
        }
    }
    return $count;
}

/**
 * Copy a whole folder
 * @param string $source Current path
 * @param string $target Target path
 * @return bool 
 */
function full_copy( $source, $target ) {
	if ( is_dir( $source ) ) {
            
		if(!mkdir( $target ))
                {
                    return false;
                }
                
		$d = dir( $source );
		while ( FALSE !== ( $entry = $d->read() ) ) {
			if ( $entry == '.' || $entry == '..' ) {
				continue;
			}
			$Entry = $source . '/' . $entry; 
			if ( is_dir( $Entry ) ) {
				full_copy( $Entry, $target . '/' . $entry );
				continue;
			}
			if(!copy( $Entry, $target . '/' . $entry ))
                        {
                            return false;
                        }
		}
 
		$d->close();
                return true;
	}else {
		if(!copy( $source, $target ))
                {
                    return false;
                }
                else{return true;}
	}
}



////// END FILE FUNCTIONS //////

// Course we are importing from
$importfrom = optional_param('importfrom', 0, PARAM_INT);
// The dir
$wdir = optional_param('wdir', "/", PARAM_TEXT);

/* Nasty little hack to fix a backslash problem in the directories/files listing */
if($wdir == '\\\\')
{
    $wdir = "/";
}
if(preg_match("/^\/\//", $wdir))
{
    $wdir = str_replace("//", "/", $wdir);
}
/* End of nasty little hack */

global $USER, $CFG;

$user = get_record('user', 'id', $userid);
$username = $user->username;

$title = get_string('importmyfiles', 'block_my_files');
$navlinks[] = array('name' => $title, 'link' => "$CFG->wwwroot/blocks/my_files/import.php?courseid=".$courseid, 'type' => 'misc');
$navlinks[] = array('name' => $course->shortname, 'link' => "$CFG->wwwroot/course/view.php?id=".$courseid, 'type' => 'misc');
$navlinks[] = array('name' => $user->firstname . ' ' . $user->lastname, 'link' => false, 'type' => 'misc');
$navigation = build_navigation($navlinks);

print_header_simple($title, '', $navigation);

$output = "";

$output .= "<div style='padding:10px;'>";
$output .= "<br>";

$output .= "<h2 style='text-align:center;'>".get_string('importinto', 'block_my_files'). " " . htmlentities($course->fullname, ENT_QUOTES)."</h2>";
$output .= "<br>";
$output .= "<h3>Select a Course to Import From:</h3>";

// Get a list of courses this user is an editing teacher on
$sql = "SELECT c.*
        FROM mdl_course c
        JOIN mdl_context ct ON ct.instanceid = c.id
        JOIN mdl_role_assignments r ON r.contextid = ct.id
        WHERE r.userid = '".$user->id."'
        AND r.roleid = 3
        AND c.id <> '".$courseid."'";

$list_courses = get_records_sql($sql);

// Display a link for each course
foreach($list_courses AS $courses)
{
    $output .= "<a href='import.php?courseid=".$courseid."&importfrom=".$courses->id."'>".$courses->fullname."</a><br>";
}

$output .= "<br><br>";



    // Check if course directory exists in course we are trying to import to, if not create it
    if(!file_exists($CFG->dataroot . '/' . $course->id))
    {
        if(!make_upload_directory($course->id))
        {
            error("Course Directory Doesn't Exist & Failed To Create It");
        }
    }



// Check if they have selected a course to import from, and if so, display a list of all the files in that course
$importfrom = (string)$importfrom;

// If valid id format for importfrom (digit)
if(ctype_digit($importfrom) && !empty($importfrom))
{
    
    // Check if it's a valid course and that this user is an editing teacher on it
    if(!$import_course = get_record('course', 'id', $importfrom))
    {
        error("Invalid Course ID To Import From");
    }

    if(!$import_course_context = get_context_instance(CONTEXT_COURSE, $import_course->id))
    {
        error("Invalid Course Context");
    }
    
    if(!has_capability('block/my_files:view',$import_course_context))
    {
        error("You Do Not Have The Correct Permissions To Import From That Course");
    }
    
    // Find all the files that have been uploaded to this course
    if ($wdir == '') {
        $wdir = "/";
    }
    
    if ($wdir{0} != '/') {  //make sure $wdir starts with slash
        $wdir = "/".$wdir;
    }
    
    $directory = $CFG->dataroot;
    $courseDirectory = $directory . "/" . $import_course->id . $wdir;
    
    // Check if course directory exists in course we are importing from, if not create it
    if(!file_exists($directory . '/' . $import_course->id))
    {
        if(!make_upload_directory($import_course->id))
        {
            error("Course Directory Doesn't Exist & Failed To Create It");
        }
    }
    
       
    // Check if directory actually exists that we are trying to access
    if(!file_exists($courseDirectory))
    {
        error("No Such Directory In This Course");
    }
    
    
    /* Before we go about listing the files, let's check if the form is submitted */
    
    if(isset($_POST['importFiles']))
    {
        
        // If filelist is set
        if(setfilelist($_POST))
        {
            
            $theFileList = $USER->filelist;
            
            // If the filelist we are trying to import is > 0
            if(count($theFileList) > 0 && confirm_sesskey())
            {
                
                $newCourseDirectory = $directory . "/" . $courseid . "/";
                foreach($USER->filelist AS $file):
                    $shortfile = basename($file);
                    $oldfile = $courseDirectory.$file;
                    $newfile = $newCourseDirectory.$shortfile;
                    
                    // Copy a file
                    if(is_file($oldfile))
                    {
                    
                        if (!copy($oldfile, $newfile)) {
                            // Failed
                            $output .= "<p style='color:red;text-align:center;'>Error: $shortfile not moved</p>";
                        }else{
                            // Success
                            $output .= "<p style='color:green;text-align:center;'>Success: $shortfile imported to ".$import_course->fullname."</p>";
                        }
                    
                    }
                    // Copy a directory
                    elseif(is_dir($oldfile))
                    {
                        if(!full_copy($oldfile, $newfile))
                        {
                            // Failed
                            $output .= "<p style='color:red;text-align:center;'>Error: $shortfile could not be moved</p>";
                        }else{
                            // Success
                            $output .= "<p style='color:green;text-align:center;'>Success: $shortfile was imported to ".$import_course->fullname."</p>";
                        }
                    }
                    
                endforeach;
                
            }
    
    
    
    
    
        }
    }
    
    
   // Now let's actually get the list of files in the course directory, to display
    $directory = opendir($courseDirectory);  
    $dirlist = array();
    $filelist = array();
    
    while (false !== ($file = readdir($directory))) {
        if ($file == "." || $file == "..") {
            continue;
        }

        if (is_dir($courseDirectory."/".$file)) {
            $dirlist[] = $file;
        } else {
            $filelist[] = $file;
        }
    }
    closedir($directory);
    
    if ($wdir != "/") {
        $dirlist[] = '..';
    }
    
    
    // Sort them
    if(count($dirlist) > 0){
        asort($dirlist);
    }
    if(count($filelist) > 0){
        asort($filelist);
    }
    
    $count = 0;
    
    $strname = get_string("name");
    $strsize = get_string("size");
    $strmodified = get_string("modified");
    $strfolder = get_string("folder");
    $strfile   = get_string("file");
    
    
    
    $output .= "<div style='text-align:center;'>";
    $output .= "<form action='' method='post'>";
    $output .= $import_course->fullname;
    $output .= "<hr style='width:600px;'>";
    $output .= "<table style=\"text-align:left;width:600px;margin:auto;\" class=\"files\">";
    $output .= "<tr>";
    $output .= "<th class=\"header\" scope=\"col\"></th>";
    $output .= "<th class=\"header name\" scope=\"col\" style=\"text-align:center;\">$strname</th>";
    $output .= "<th class=\"header size\" scope=\"col\">$strsize</th>";
    $output .= "<th class=\"header date\" scope=\"col\">$strmodified</th>";
    $output .= "</tr>\n";
    
        // Loop folders
        foreach($dirlist AS $dir):
            
            
              
            $output .= "<tr class=\"folder\">";
            
                if($dir == '..')
                {
                    // Is the parent folder, so don't give the option to import, that would be crazy
                    $fileurl = rawurlencode(dirname($wdir));
                    $output .= "<td>&nbsp;</td><td><a href='import.php?courseid=$courseid&importfrom=$importfrom&wdir=$fileurl'><img src=\"".$CFG->pixpath."/f/parent.gif\" class=\"icon\" alt=\"$strfolder\" />&nbsp;&nbsp;".get_string('parentfolder', 'block_my_files')."</a></td><td>&nbsp;</td><td>&nbsp;</td>";
                }
                elseif($dir == 'moddata' || $dir == 'backupdata')
                {
                    // Is moddate or backupdata, so don't give the option to import
                    $fileurl = rawurlencode($wdir."/".$dir);
                    $output .= "<td>&nbsp;</td><td><a href='import.php?courseid=$courseid&importfrom=$importfrom&wdir=$fileurl'><img src=\"".$CFG->pixpath."/f/folder.gif\" class=\"icon\" alt=\"$strfolder\" />&nbsp;&nbsp;".htmlspecialchars($dir)."</a></td><td>&nbsp;</td><td>&nbsp;</td>";
                }
                else
                {
                    // A folder we can import
                    $count++;
                    $filename = $courseDirectory."/".$dir;
                    $fileurl  = rawurlencode($wdir."/".$dir);
                    $filesafe = rawurlencode($dir);
                    $filesize = display_size(get_directory_size("$courseDirectory/$dir"));
                    $filedate = userdate(filemtime($filename), get_string("strftimedatetime"));
                    $output .= "<td><input type='checkbox' name='file$count' value='$fileurl' /></td><td><a href='import.php?courseid=$courseid&importfrom=$importfrom&wdir=$fileurl'><img src=\"".$CFG->pixpath."/f/folder.gif\" class=\"icon\" alt=\"$strfolder\" />&nbsp;&nbsp;".htmlspecialchars($dir)."</a></td><td>".$filesize."</td><td>".$filedate."</td>";
                }
            $output .= "</tr>";
            
        endforeach;
        
        // Loop files
        foreach($filelist AS $file):
            
            $count++;
            $icon = mimeinfo("icon", $file);
            $filename    = $courseDirectory."/".$file;
            $fileurl     = trim($wdir, "/")."/$file";
            $filesafe    = rawurlencode($file);
            $fileurlsafe = rawurlencode($fileurl);
            $filedate    = userdate(filemtime($filename), get_string("strftimedatetime"));
            $file_size = filesize($filename);

            
            $output .= "<tr class=\"file\">";
        
                $output .= "<td><input type='checkbox' name='file$count' value='$fileurl' /></td><td><a href='import.php?courseid=$courseid&importfrom=$importfrom&wdir=$fileurl'><img src=\"".$CFG->pixpath."/f/$icon\" class=\"icon\" alt=\"$strfile\" />&nbsp;&nbsp;".htmlspecialchars($file)."</a></td><td>".display_size($file_size)."</td><td>".$filedate."</td>";
            
            $output .= "</tr>";
            
        endforeach;
    
    $output .= "</table>";
    $output .= "<hr style='width:600px;'>";
    $output .= "<input type='hidden' name='sesskey' value='".$USER->sesskey."'/>";
    $output .= "<input type='submit' name='importFiles' value='Import Files' /><br>";
    $output .= "<strong>Please Note: If any of these folders/files have the same name as current folders/files in this course, the old ones will be overwritten by these new ones.</strong>";
    $output .= "</div>";
    
    
    
}

$output .= "</div>";

echo $output;

print_footer();
?>